<?php
/**
 * Flexpage Activity Block
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://opensource.org/licenses/gpl-3.0.html.
 *
 * @copyright Copyright (c) 2009 Moodlerooms Inc. (http://www.moodlerooms.com)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU Public License
 * @package block_flexpagemod
 * @author Mark Nielsen
 */

/**
 * Display code for block's content
 *
 * @throws coding_exception
 * @author Mark Nielsen
 * @package block_flexpagemod
 */
class block_flexpagemod_lib_mod {
    /**
     * Determine if the default rendering was used or not
     *
     * @var bool
     */
    protected $defaultused = false;

    /**
     * @var cm_info
     */
    protected $cm;

    /**
     * @var block_flexpagemod
     */
    protected $block;

    /**
     * @param cm_info $cm The course module being displayed
     * @param block_flexpagemod $block The block instance being used for displayed
     */
    public function __construct(cm_info &$cm, block_flexpagemod &$block) {
        $this->cm    = $cm;
        $this->block = $block;
    }

    /**
     * Factory: create the most relevant instance
     *
     * @static
     * @throws coding_exception
     * @param cm_info $cm The course module being displayed
     * @param block_flexpagemod $block The block instance being used for displayed
     * @return block_flexpagemod_lib_mod
     */
    public static function factory(cm_info &$cm, block_flexpagemod &$block) {
        global $CFG;

        $paths = array(
            "$CFG->dirroot/mod/$cm->modname/flexpage.php" => "mod_{$cm->modname}_flexpage",
            "$CFG->dirroot/blocks/flexpagemod/lib/mod/$cm->modname.php" => "block_flexpagemod_lib_mod_$cm->modname",
        );

        foreach ($paths as $path => $class) {
            if (!class_exists($class) and file_exists($path)) {
                require_once($path);

                // Try to help out
                if (!class_exists($class)) {
                    throw new coding_exception("Failed to find $class");
                }
            }
            if (class_exists($class)) {
                $instance = new $class($cm, $block);

                // Try to help out
                if (!$instance instanceof block_flexpagemod_lib_mod) {
                    throw new coding_exception("The class $class must extend block_flexpagemod_lib_mod");
                }
                return $instance;
            }
        }
        return new block_flexpagemod_lib_mod($cm, $block);
    }

    /**
     * @return cm_info
     */
    public function get_cm() {
        return $this->cm;
    }

    /**
     * @return block_flexpagemod
     */
    public function get_block() {
        return $this->block;
    }

    /**
     * Setup $this->block for display
     *
     * @return void
     */
    public function setup_block() {
        // Check if we are not visible to the user
        if (!$this->get_cm()->uservisible) {
            // If we have availability information, we do default display
            if ($this->get_cm()->showavailability and !empty($this->get_cm()->availableinfo)) {
                $this->default_block_setup();
            }
        } else {
            // Allow module custom display
            $this->module_block_setup();
            $this->add_mod_commands();
        }
    }

    /**
     * Append text to the block's main content area
     *
     * @param string $content
     * @return block_flexpagemod_lib_mod
     */
    public function append_content($content) {
        $this->get_block()->content->text .= $content;
        return $this;
    }

    /**
     * Append text to the block's footer area
     *
     * @param string $content
     * @return block_flexpagemod_lib_mod
     */
    public function append_footer($content) {
        $this->get_block()->content->footer .= $content;
        return $this;
    }

    /**
     * Add module commands when not using default rendering
     *
     * Must be called after block text has been completely filled.
     *
     * @return void
     */
    public function add_mod_commands() {
        global $PAGE;

        if (!$this->defaultused and $PAGE->user_is_editing()) {
            $mod = $this->get_cm();
            $course = $this->get_block()->page->course;
            $groupbuttons = ($course->groupmode or (!$course->groupmodeforce));
            $groupbuttonslink = (!$course->groupmodeforce);

            if ($groupbuttons and plugin_supports('mod', $mod->modname, FEATURE_GROUPS, 0)) {
                if (!$mod->groupmodelink = $groupbuttonslink) {
                    $mod->groupmode = $course->groupmode;
                }
            } else {
                $mod->groupmode = false;
            }
            $buttons = make_editing_buttons($mod, true, true, $mod->indent, $mod->sectionnum);
            $buttons = html_writer::tag('div', $buttons, array('class' => 'block_flexpagemod_commands'));

            $this->get_block()->content->text = html_writer::tag(
                'div',
                $buttons.$this->get_block()->content->text,
                array('class' => 'block_flexpagemod_commands_wrapper')
            );
        }
    }

    /**
     * Customized block setup for a particular module
     *
     * @return void
     */
    public function module_block_setup() {
        $this->default_block_setup();
    }

    /**
     * Default block setup, make it look like a link from topics/weeks format
     *
     * @return void
     */
    public function default_block_setup() {

        require_once(__DIR__.'/section.php');

        // Mark our flag
        $this->defaultused = true;

        $cm = $this->get_cm();
        $cm->modfullname = get_string('modulename', $cm->modname);

        $course     = $this->get_block()->page->course;
        $mods       = array($cm->id => $cm);
        $section    = $cm->get_modinfo()->get_section_info($cm->sectionnum);
        $sectionlib = new block_flexpagemod_lib_seciton();

        ob_start();
        $sectionlib->print_section($course, $section, $mods, array());
        $output = ob_get_contents();
        ob_end_clean();

        if (!empty($output)) {
            $output = $this->inject_attribute('ul', 'role', 'presentation', $output);
            $output = $this->inject_attribute('li', 'role', 'presentation', $output);

            $this->append_content(
                html_writer::tag('div', $output, array('class' => 'block_flexpagemod_default course-content'))
            );
        }
    }

    /**
     * Searches for the first occurrence of a tag and injects
     * an HTML attribute tag.
     *
     * @param string $tag The tag to search for
     * @param string $name The attribute name
     * @param string $value The attribute value
     * @param string $haystack The haystack to operate on
     * @return mixed
     */
    public function inject_attribute($tag, $name, $value, $haystack) {
        $open = '<'.$tag;
        $newopen = "<$tag $name=\"$value\" ";

        $start = stripos($haystack, $open);
        if ($start === false) {
            return $haystack;
        }
        return substr_replace($haystack, $newopen, $start, strlen($open));
    }
}